@@ -91,10 +91,17 @@ class LossAggregation:
         # self.copulaPar_la = kwargs['copulaPar_la']
         ## marginals
         ##available marginals
-        self.mdict = {'gamma': stats.gamma,
-                      'lognorm': stats.lognorm,
-                      'exponential': distributions.exponential,
-                      'genpareto': stats.genpareto}
+        self.mdict = {'gamma': distributions.Gamma,
+                  'lognorm': distributions.Lognorm,
+                  'exponential':distributions.Exponential,
+                  'genpareto': distributions.Genpareto,
+                  'burr12': distributions.Burr12,
+                  'dagum': distributions.Dagum,
+                  'invgamma': distributions.Invgamma,
+                  'weibull_min': distributions.Weibull_min,
+                  'invweibull': distributions.Invweibull,
+                  'beta': distributions.Beta}
+
         ### distributions
         self.m1 = kwargs['m1']
         self.m2 = kwargs['m2']
@@ -162,13 +169,7 @@ class LossAggregation:
         try:
             self.copula=self.copdist(**self.coppar)
         except:
-            logger.error("The copula distribution is not parametrized correctly \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html ")
-
-        # self.__copuladict={'clayton':copulae.ClaytonCopula,'gumbel':hfns.GumbelCDF}
-        # try:
-        #     self.__copulaCDF=self.copdist(**self.coppar).cdf
-        # except:
-        #     logger.error("The copula distribution is not parametrized correctly \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html ")
+            logger.error("The copula distribution is not parametrized correctly \n See our documentation: https://gem-analytics.github.io/gemact/ ")
 
 
         ## about the AEP algorithm
@@ -232,7 +233,7 @@ class LossAggregation:
         cdict = {'clayton': hfns.ClaytonCDF,
                   'frank': hfns.FrankCDF,
                   'gumbel':hfns.GumbelCDF}
-        assert var in cdict.keys(), "%r is not supported \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html " % var
+        assert var in cdict.keys(), "%r is not supported \n See our documentation: https://gem-analytics.github.io/gemact/" % var
         self.__copdist = cdict[var]
 
     @property
@@ -256,7 +257,7 @@ class LossAggregation:
 
     @m1.setter
     def m1(self, var):
-        assert var in self.mdict.keys(), "%r is not supported \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html " % var
+        assert var in self.mdict.keys(), "%r is not supported \n See our documentation: https://gem-analytics.github.io/gemact/" % var
         self.__m1 = self.mdict[var]
 
     @property
@@ -265,7 +266,7 @@ class LossAggregation:
 
     @m2.setter
     def m2(self, var):
-        assert var in self.mdict.keys(), "%r is not supported \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html " % var
+        assert var in self.mdict.keys(), "%r is not supported \n See our documentation: https://gem-analytics.github.io/gemact/" % var
         self.__m2 = self.mdict[var]
 
     @property
@@ -275,7 +276,7 @@ class LossAggregation:
     @m3.setter
     def m3(self, var):
         if var is not None:
-            assert var in self.mdict.keys(), "%r is not supported \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html " % var
+            assert var in self.mdict.keys(), "%r is not supported \n See our documentation: https://gem-analytics.github.io/gemact/ " % var
             self.__m3 = self.mdict[var]
         else:
             assert var is None, logger.error('Please provide in m3 a marginal supported distribution or drop the argument.')
@@ -288,7 +289,7 @@ class LossAggregation:
     @m4.setter
     def m4(self, var):
         if var is not None:
-            assert var in self.mdict.keys(), "%r is not supported \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html " % var
+            assert var in self.mdict.keys(), "%r is not supported \n See our documentation: https://gem-analytics.github.io/gemact/" % var
             self.__m4 = self.mdict[var]
         else:
             assert var is None, logger.error(
@@ -302,7 +303,7 @@ class LossAggregation:
     @m5.setter
     def m5(self, var):
         if var is not None:
-            assert var in self.mdict.keys(), "%r is not supported \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html " % var
+            assert var in self.mdict.keys(), "%r is not supported \n See our documentation: https://gem-analytics.github.io/gemact/" % var
             self.__m5 = self.mdict[var]
         else:
             assert var is None, logger.error(
