@@ -8,20 +8,19 @@ from twiggy import quick_setup,log
 quick_setup()
 logger = log.name('lossmodel')
 
-
 ## Actual
 class Frequency:
     """
     This class computes the frequency underlying the collective risk model.
 
-    :param fdist: Frequency distribution. Provided as a string, it is then set to a discrete distribution in the setter property.
-    :type fdist: ``str``
-    :param fpar: It contains the frequency distribution parameters. Frequency distributions must be parametrized according to the scipy parametrizaton.
-    :type fpar: ``dict``
-    :param fq: Frequency distribution object.
-    :type fq: ``scipy.stats._distn_infrastructure.rv_frozen``
-    :param p0: When the frequency distribution is a ZM distribution, this is the mixture probability.
-    :type p0: ``float``
+    :param \**kwargs:
+    See below
+
+    :Keyword Arguments:
+        * *fdist* (``str``) --
+          Name of the frequency distribution. See the distribution module for the discrete distributions supported in GEMAct.
+        * *fpar* (``dict``) --
+          parameters of the frequency distribution.
 
     """
 
@@ -36,12 +35,23 @@ class Frequency:
         try:
             self.fq = self.fdist(**self.fpar)
         except:
-            logger.error("The frequency distribution is not parametrized correctly \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html ")
+            logger.error("The frequency distribution is not parametrized correctly \n See our documentation: https://gem-analytics.github.io/gemact/")
 
         if self.fdist.name in ['ZMpoisson','ZMbinom','ZMgeom','ZMnbinom']:
             self.p0= self.fpar['p0M']
         else:
             self.p0=None
+    #
+    #
+    # @property
+    # def stvpar(self):
+    #     return self.__stvpar
+    #
+    # @stvpar.setter
+    # def stvpar(self, var):
+    #     if not var is None:
+    #         assert isinstance(var,dict), logger.error('The structure variable distribution parameters must be given as a dictionary')
+    #     self.__stvpar = var
 
     @property
     def fdist(self):
@@ -50,23 +60,38 @@ class Frequency:
     @fdist.setter
     def fdist(self, var):
         #this is the set of possible distributions
-        frdict = {'poisson': distributions.poisson,
+        frdict = {'poisson': distributions.Poisson,
                   'ZTpoisson':distributions.ZTpoisson,
                   'ZMpoisson': distributions.ZMpoisson,
-                  'binom': distributions.binom,
+                  'binom': distributions.Binom,
                   'ZTbinom': distributions.ZTbinom,
                   'ZMbinom': distributions.ZMbinom,
-                  'geom':distributions.geom,
+                  'geom':distributions.Geom,
                   'ZTgeom': distributions.ZTgeom,
                   'ZMgeom': distributions.ZMgeom,
-                  'nbinom':distributions.nbinom,
+                  'nbinom':distributions.Nbinom,
                   'ZTnbinom': distributions.ZTnbinom,
                   'ZMnbinom': distributions.ZMnbinom
                   }
 
-        assert var in frdict.keys(), "%r is not supported \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html " % var
+        assert var in frdict.keys(), "%r is not supported \n See our documentation: https://gem-analytics.github.io/gemact/" % var
+        # if self.stvpar is None:
         self.__fdist = frdict[var]
+        # else:
+        # assert var in ['poisson','ZTpoisson','ZMpoisson'], logger.error('Models with structure variable are available only for poisson, ZTpoisson and ZMpoisson distributions')
+        # if var == 'poisson':
+        #     self.__fdist = frdict['nbinom']
+        # elif var == 'ZTpoisson':
+        #     logger.info(
+        #         "The zero-truncation with the presence of a structure variable is applied to the a posteriori model")
+        #     self.__fdist = frdict['ZTnbinom']
+        # elif var == 'ZMpoisson':
+        #     logger.info(
+        #         "The zero-modification with the presence of a structure variable is applied to the a posteriori model")
+        #     self.__fdist = frdict['ZMnbinom']
 
+            #caso binomiale e geometrica dobbiamo parlare perchè nel caso serve ricavare tutte le relative ab0 o scegliere di non portare
+            #recurrent e fft
 
     @property
     def fpar(self):
@@ -79,6 +104,14 @@ class Frequency:
             if not 'loc' in var.keys():
                 var['loc'] = -1
         self.__fpar = var
+        # if self.stvpar is None:
+        #     self.__fpar = var
+        # else:
+        #         if self.fdist in ['poisson','ZMpoisson','ZTpoisson']: ## modificato
+        #             try:
+        #                 self.__fpar={'n':self.stvpar['a'],'p':self.stvpar['scale']/(1+self.stvpar['scale'])}
+        #             except:
+        #                 logger.error("You did not parametrize the distribution correctly. Check the gamma parameters are provided as 'a' and 'scale'" )
 
     @property
     def p0(self):
@@ -100,6 +133,46 @@ class Frequency:
         else:
             self.__p0 = var
 
+    # def fgp(self,f,a=0,b=0):
+    #     """
+    #     It computes the probability generating function of the discrete distribution given the (a,b,1) parametrization.
+    #
+    #     :param f: argument of the probability generating function.
+    #               It will be severity discrete distribution when computing the collective risk model.
+    #     :type f: numpy.ndarray
+    #     :param a: a parameter of the distribution.
+    #     :param b: b parameter of the distribution.
+    #     :return: probability generated in f.
+    #     """
+    #
+    #     if self.fdist.name == 'poisson':
+    #         return np.exp(b*(f-1))
+    #     if self.fdist.name== 'ZTpoisson':
+    #         return (np.exp(b*f)-1)/(np.exp(b)-1)
+    #     if self.fdist.name== 'ZMpoisson':
+    #         return self.p0+(1-self.p0)*((np.exp(b*f)-1)/(np.exp(b)-1))
+    #
+    #     if self.fdist.name =='binom':
+    #         return (1+a/(a-1)*(f-1))**(-b/a-1)
+    #     if self.fdist.name =='ZTbinom':
+    #         return ((1+a/(a-1)*(f-1))**(-b/a-1)-(1-a)**(b/a+1))/(1-(1-a)**(b/a+1))
+    #     if self.fdist.name== 'ZMbinom':
+    #         return self.p0+(1-self.p0)*((1+a/(a-1)*(f-1))**(-b/a-1)-(1-a)**(b/a+1))/(1-(1-a)**(b/a+1))
+    #
+    #     if self.fdist.name =='geom':
+    #         return (1-a/(1-a)*(f-1))**(-1)
+    #     if self.fdist.name =='ZTgeom':
+    #         return (1/(1-(f-1)/(1-a))-1+a)/a
+    #     if self.fdist.name == 'ZMgeom':
+    #         return self.p0+(1-self.p0)*(1/(1-(f-1)/(1-a))-1+a)/a
+    #
+    #     if self.fdist.name =='nbinom':
+    #         return (1-a/(1-a)*(f-1))**(-b/a-1)
+    #     if self.fdist.name =='ZTnbinom':
+    #         return ((1/(1-(f-1)*a/(1-a)))**(b/a+1)-(1-a)**(b/a+1))/(1-(1-a)**(b/a+1))
+    #     if self.fdist.name == 'ZMnbinom':
+    #         return self.p0+(1-self.p0)*((1/(1-(f-1)*a/(1-a)))**(b/a+1)-(1-a)**(b/a+1))/(1-(1-a)**(b/a+1))
+
     def abp0g0(self,fj):
         """
         It returns the parameters of the discrete distribution according to the (a,b,k) parametrization,
@@ -113,6 +186,84 @@ class Frequency:
         a,b,p0=self.fq.abk()
         return a,b,p0,[self.fq.pgf(fj[0])]
 
+        # if self.fdist.name in ['poisson', 'binom', 'geom', 'nbinom'] and (self.p0 is not None):
+        #     logger.info(
+        #         "The parameter p0 is arbitrary only for Zero-Modified distributions. \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html ")
+        #
+        # # it returns a,b, p0,g
+        # if self.fdist.name == 'poisson':
+        #     return 0, \
+        #            self.fpar['mu'], \
+        #             np.array(np.exp(-self.fpar['mu'])),\
+        #            [self.fgp(f=fj[0],a=0,b=self.fpar['mu'])]
+        #
+        # if self.fdist.name == 'ZTpoisson':
+        #     return 0, \
+        #            self.fpar['mu'], \
+        #            0,\
+        #            [self.fgp(f=fj[0],a=0,b=self.fpar['mu'])]
+        #
+        # if self.fdist.name == 'ZMpoisson':
+        #     return 0, \
+        #            self.fpar['mu'], \
+        #            self.p0, \
+        #            [self.fgp(f=fj[0],a=0,b=self.fpar['mu'])]
+        #
+        # if self.fdist.name == 'binom':
+        #     return -self.fpar['p']/(1-self.fpar['p']), \
+        #            (self.fpar['n']+1)*(self.fpar['p']/(1-self.fpar['p'])),\
+        #            (1-self.fpar['p'])**self.fpar['n'],\
+        #            [self.fgp(f=fj[0],a=-self.fpar['p']/(1-self.fpar['p']),b=(self.fpar['n']+1)*(self.fpar['p']/(1-self.fpar['p'])))]
+        #
+        # if self.fdist.name == 'ZTbinom':
+        #     return -self.fpar['p'] / (1 - self.fpar['p']), \
+        #            (self.fpar['n'] + 1) * (self.fpar['p'] / (1 - self.fpar['p'])), \
+        #            0,\
+        #            [self.fgp(f=fj[0],a=-self.fpar['p']/(1-self.fpar['p']),b=(self.fpar['n']+1)*(self.fpar['p']/(1-self.fpar['p'])))]
+        #
+        # if self.fdist.name == 'ZMbinom':
+        #     return -self.fpar['p'] / (1 - self.fpar['p']), \
+        #            (self.fpar['n'] + 1) * (self.fpar['p'] / (1 - self.fpar['p'])), \
+        #            self.p0,\
+        #            [self.fgp(f=fj[0],a=-self.fpar['p']/(1-self.fpar['p']),b=(self.fpar['n']+1)*(self.fpar['p']/(1-self.fpar['p'])))]
+        #
+        # if self.fdist.name == 'geom':
+        #     return 1-self.fpar['p'], \
+        #            0,\
+        #            np.array([((1-self.fpar['p'])/self.fpar['p'])**-1]), \
+        #            [self.fgp(f=fj[0], a=1-self.fpar['p'],b=0)]
+        #
+        # if self.fdist.name == 'ZTgeom':
+        #     return 1-self.fpar['p'], \
+        #            0,\
+        #            np.array([0]),\
+        #            [self.fgp(f=fj[0], a=1-self.fpar['p'],b=0)]
+        #
+        # if self.fdist.name == 'ZMgeom':
+        #     return 1-self.fpar['p'], \
+        #            0, \
+        #            np.array([self.p0]), \
+        #            [self.fgp(f=fj[0], a=1-self.fpar['p'],b=0)]
+        #
+        # if self.fdist.name == 'nbinom':
+        #     return 1-self.fpar['p'], \
+        #            (self.fpar['n']-1)*(1-self.fpar['p']), \
+        #            np.array([self.fpar['p']**self.fpar['n']]), \
+        #            [self.fgp(f=fj[0], a=1-self.fpar['p'],b=(self.fpar['n']-1)*(1-self.fpar['p']))]
+        #
+        # if self.fdist.name == 'ZTnbinom':
+        #     return 1-self.fpar['p'], \
+        #            (self.fpar['n']-1)*(1-self.fpar['p']),\
+        #            np.array([0]), \
+        #            [self.fgp(f=fj[0], a=1 - self.fpar['p'], b=(self.fpar['n'] - 1) * (1 - self.fpar['p']))]
+        #
+        # if self.fdist.name == 'ZMnbinom':
+        #     return 1-self.fpar['p'], \
+        #            (self.fpar['n']-1)*(1-self.fpar['p']),\
+        #            np.array([self.p0]), \
+        #            [self.fgp(f=fj[0], a=1 - self.fpar['p'], b=(self.fpar['n'] - 1) * (1 - self.fpar['p']))]
+
+
 class Severity:
     """
     Computes the severity distribution of the underlying collective risk model.
@@ -123,17 +274,16 @@ class Severity:
     :type m: ``int``
     :param d: deductible, it is set to zero when not present.
     :type d: ``int`` or ``float``
-    :param u:
+    :param u: upper priority, individual.
     :type u: ``int`` or ``float``
     :param \**kwargs:
         See below
 
     :Keyword Arguments:
-        * *sdist* (``list``) --
-          Extra stuff
+        * *sdist* (``str``) --
+          Name of the severity distribution. See the distribution module for the continuous distributions supported in GEMAct.
         * *spar* (``dict``) --
-          Additional content
-
+          parameters of the severity distribution.
     """
     def __init__(self, h=None, m=None,d=0, u=float('inf'),**kwargs):
 
@@ -154,7 +304,7 @@ class Severity:
             self.sv = self.sdist(**self.spar)
         except:
             print(
-                "The severity distribution is not parametrized correctly \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html ")
+                "The severity distribution is not parametrized correctly \n See our documentation: https://gem-analytics.github.io/gemact/ ")
 
 
     @property
@@ -164,17 +314,17 @@ class Severity:
     @sdist.setter
     def sdist(self, var):
         # this is the set of possible distributions
-        svdict = {'gamma': distributions.gamma,
-                  'lognorm':distributions.lognorm,
-                  'exponential':distributions.exponential,
-                  'genpareto': distributions.genpareto,
-                  'burr12': distributions.burr12,
-                  'dagum': distributions.dagum,
-                  'invgamma': distributions.invgamma,
-                  'weibull_min':distributions.weibull_min,
-                  'invweibull':distributions.invweibull,
-                  'beta':distributions.beta}
-        assert var in svdict.keys(), "%r is not supported \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html " % var
+        svdict = {'gamma': distributions.Gamma,
+                  'lognorm':distributions.Lognorm,
+                  'exponential':distributions.Exponential,
+                  'genpareto': distributions.Genpareto,
+                  'burr12': distributions.Burr12,
+                  'dagum': distributions.Dagum,
+                  'invgamma': distributions.Invgamma,
+                  'weibull_min':distributions.Weibull_min,
+                  'invweibull':distributions.Invweibull,
+                  'beta':distributions.Beta}
+        assert var in svdict.keys(), "%r is not supported \n See our documentation: https://gem-analytics.github.io/gemact/ " % var
         self.__sdist = svdict[var]
 
     @property
@@ -257,7 +407,13 @@ class Severity:
                     1 - self.sv.cdf(self.d)))
         if self.u != float('inf'):
             fj = np.append(fj, (1 - self.sv.cdf(self.u - self.h / 2)) / (1 - self.sv.cdf(self.d)))
-        return fj
+
+        seq=self.loc_ + np.arange(0, self.m) * self.h
+
+        if self.u!= float('inf'):
+            seq=np.concatenate((seq,[seq[-1]+self.h]))
+
+        return {'fj':fj,'severity_seq':seq}
 
     def __umhoverd(self):
         """
@@ -272,6 +428,7 @@ class Severity:
         else:
             return (self.sv.Emv(self.u - self.loc_)-self.sv.Emv(self.u - self.loc_ - self.h))/(self.h*self.sv.den(d=self.d,loc=self.loc_))
 
+
     def localMoments(self):
         """
         Severity discretization according to the local moments method.
@@ -280,7 +437,6 @@ class Severity:
         :rtype: ``numpy.ndarray``
 
         """
-
         v_ = self.__umhoverd()
         n = self.sv.Emv(self.d + self.h - self.loc_) - self.sv.Emv(self.d - self.loc_)
         den=self.h*self.sv.den(d=self.d,loc=self.loc_)
@@ -290,7 +446,11 @@ class Severity:
 
         ## keep the following two lines outside the if cycles
         fj = np.append(1 - n / den, nj / den)
-        return np.append(fj, v_)
+
+        seq=self.loc_ + np.arange(0, self.m) * self.h
+        if self.u!= float('inf'):
+            seq=np.concatenate((seq,[seq[-1]+self.h]))
+        return {'fj':np.append(fj, v_), 'severity_seq': seq}
 
 
 class LossModel(Frequency,Severity):
@@ -298,13 +458,13 @@ class LossModel(Frequency,Severity):
     It computes the collective risk model given the frequency and severity assumptions.
     It allows for (re)insurance pricing and risk modeling.
 
-    :param nsim: number of simulations, MC and QMC.
+    :param method: computational method for the aggregate cost. Available choices are Fast Fourier Transform ('fft'), recursive method ('recursive') and Monte Carlo simulation ('mcsimulation').
+    :type method: ``str``
+    :param nsim: number of simulations, MC.
     :type nsim: ``int``
-    :param skip: number of values to be dropped from the QMC simulated sequence.
-    :type skip: ``int``
     :param tilt: whether tilting is present or not when computing the discrete Fourier transform.
     :type tilt: ``bool`` or ``float``
-    :param setseed: seed parameter for MC and QMC.
+    :param setseed: seed parameter for MC.
     :type setseed: ``int``
     :param discretizationmethod:  discretization method for the severity. Default is 'localmoments'.
     :type discretizationmethod: ``str``
@@ -319,10 +479,31 @@ class LossModel(Frequency,Severity):
     :param alphaqs: quota share.
     :type alphaqs: ``float``
 
+    Inherits from Frequency:
+
+    :param fdist: name of the frequency distribution. See the distributions module for the discrete distributions supported in GEMAct.
+    :type fdist: ``str``
+    :param fpar: parameters of the frequency distribution.
+    :type fpar: ``dict``
+
+    Inherits from Severity:
+
+    :param h: severity discretization step.
+    :type h: ``float``
+    :param m: number of points of the discrete severity.
+    :type m: ``int``
+    :param d: deductible, it is set to zero when not present.
+    :type d: ``int`` or ``float``
+    :param u: upper priority, individual.
+    :type u: ``int`` or ``float``
+    :param sdist: name of the severity distribution. See the distribution module for the continuous distributions supported in GEMAct.
+    :type sdist: ``str``
+    :param spar: parameters of the severity distribution.
+    :type spar: ``dict``
+
     """
     def __init__(self, method=None,
                  nsim=10000,
-                 skip=10,
                  tilt=True,
                  setseed=42,
                  discretizationmethod = 'localmoments',
@@ -342,7 +523,6 @@ class LossModel(Frequency,Severity):
 
         self.method = method
         self.nsim = nsim
-        self.skip= skip
         self.setseed = setseed
         self.discretizationmethod = discretizationmethod
         self.n = n
@@ -355,7 +535,7 @@ class LossModel(Frequency,Severity):
         self.lossModel = self.lossModelComputation()
 
 
-        if self.method in ['mcsimulation','qmcsimulation']:
+        if self.method in ['mcsimulation']: #,'qmcsimulation'
             self.__algtype='simulative'
         else:
             self.__algtype = 'notsimulative'
@@ -370,7 +550,8 @@ class LossModel(Frequency,Severity):
             self.__method = 'mcsimulation'
         else:
             assert isinstance(var,str), '%r is not a string' %var
-            assert var.lower() in ['mcsimulation','recursive','fft','qmcsimulation'], "%r is not supported in our package \n See our documentation: https://UnGiornoCiFacciamoPureIlSito.html " % var
+            assert var.lower() in ['mcsimulation','recursive','fft'], "%r is not supported in our package \n See our documentation: https://gem-analytics.github.io/gemact/" % var
+            # ,'qmcsimulation'
             if not var.lower() == var:
                 logger.warning("What did you mean with %r? 'method' is set to %r." % (var, var.lower()))
             self.__method = var.lower()
@@ -388,14 +569,6 @@ class LossModel(Frequency,Severity):
             logger.error('nsim=%r must be set to an integer' % var)
         self.__nsim= var
 
-    @property
-    def skip(self):
-        return self.__skip
-
-    @skip.setter
-    def skip(self, var):
-        assert isinstance(var, int), logger.error('skip=%r must be set to an integer' % var)
-        self.__skip = var
 
     @property
     def setseed(self):
@@ -513,8 +686,8 @@ class LossModel(Frequency,Severity):
             return self.__MCsimulation()
         if self.method == 'recursive':
             return self.__recursive()
-        if self.method == 'qmcsimulation':
-            return self.__QMCsimulation()
+        # if self.method == 'qmcsimulation':
+        #     return self.__QMCsimulation()
         if self.method == 'fft':
             return self.__fft()
 
@@ -572,9 +745,11 @@ class LossModel(Frequency,Severity):
         logger.info('..Computing the distribution via FFT..')
 
         if self.discretizationmethod == 'localmoments':
-            fj = self.localMoments()
+            sevdict = self.localMoments()
         else:
-            fj = self.massDispersal()
+            sevdict = self.massDispersal()
+
+        fj=sevdict['fj']
 
         if isinstance(self.tilt,bool):
             if self.tilt == True:
@@ -609,10 +784,11 @@ class LossModel(Frequency,Severity):
         logger.info('..Computing the recursive distribution..')
 
         if self.discretizationmethod == 'localmoments':
-            fj = self.localMoments()
+            sevdict = self.localMoments()
         else:
-            fj = self.massDispersal()
+            sevdict = self.massDispersal()
 
+        fj=sevdict['fj']
         a, b, p0, g = self.abp0g0(fj)
 
         if self.u == float('inf'):
@@ -667,7 +843,7 @@ class LossModel(Frequency,Severity):
         if fqsample[0:1] == 0:
             xsim = np.array([0])
         else:
-            xsim =  np.array([np.sum(svsample[0:cs[0]])])
+            xsim = np.array([np.sum(svsample[0:cs[0]])])
 
         for i in range(0, self.nsim - 1):
             if cs[i] == cs[i + 1]:
@@ -686,77 +862,99 @@ class LossModel(Frequency,Severity):
                 'ecdf':ecdf,
                 'sequence':x_}
 
-    def __QMCsimulation(self):
-        """
-        Aggregate cost computation according to the Quasi-Monte Carlo simulation.
-
-        :return: aggregate cost empirical pdf, aggregate cost empirical cdf,aggregate cost simulated values.
-        :rtype: ``dict``
-        """
-        if self.d > 1e-05:
-            p0 = self.sv.cdf(self.d)
-        else:
-            p0 = 0
-        logger.info('..QMC Simulating the aggregate distribution..')
-        np.random.seed(self.setseed)
-        fss = hfns.sobol_generator(n=self.nsim, dim=1,skip=self.skip).flatten()
-        fqsample = self.fq.ppf(fss)
-        sss = hfns.sobol_generator(n=int(np.sum(fqsample) + 1.1*np.ceil(p0 * self.nsim)), dim=1,skip=self.skip).flatten()
-        svsample = self.sv.ppf(sss)
-        if self.d > 1e-05:
-            np.random.seed(self.setseed)
-            svsample = svsample[svsample > self.d]
-            while svsample.shape[0] < self.nsim:
-                n0 = np.ceil(p0 * (self.nsim - svsample.shape[0]))
-                sss = hfns.sobol_generator(n=int(1.1*n0), dim=1,skip=self.skip).flatten()
-                svsample = np.concatenate((svsample, self.sv.ppf(sss)))
-                svsample = svsample[svsample > self.d]
-                # timeel_=round((datetime.now()-now).seconds/60)
-
-                # if timeel_%3 == 0 and timeel_!=0 and count_ ==0:
-                #     logger.warning('Pay attention: when a deductible is present, QMC simulation can be very slow.\n %s minutes elapsed, already'%str((datetime.now()-now).seconds/60))
-                #     count_=count_
-
-            svsample = svsample - self.d
-
-        if self.u < float('inf'):
-            svsample = np.minimum(svsample, self.u)
-
-        # random.seed(setseed) elimino: trovo seed nelle funzioni dopo
-        # fss = hfns.sobol_generator(n=self.nsim, dim=1).flatten() #sobol_seq.i4_sobol_generate(1, self.nsim)
-        # update frequency parameters
-        # sample frequency
-        # fqsample = self.fq.ppf(fss)
-        # sss = hfns.sobol_generator(n=int(np.sum(fqsample)), dim=1).flatten()
-        # sss = sobol_seq.i4_sobol_generate(1, int(np.sum(fqsample))).flatten()
-
-        # svsample = self.sv.ppf(sss)
-
-        cs = np.cumsum(fqsample).astype('int')
-
-        if fqsample[0:1] == 0:
-            xsim = [0]
-        else:
-            xsim = [np.sum(svsample[0:int(cs[0])])]
-
-        for i in range(0, self.nsim - 1):
-            if cs[i] == cs[i + 1]:
-                xsim.append(0)
-            else:
-                xsim.append(np.sum(svsample[int(cs[i]):int(cs[i + 1])]))
-
-        logger.info('..QMC Simulation completed..')
-
-        x_, ecdf = hfns.ecdf(xsim)
-        epdf = np.repeat(1 / self.nsim, self.nsim)
-
-        return {'epdf': epdf,
-                'ecdf':ecdf,
-                'sequence':x_}
+    # def __QMCsimulation(self):
+    #     """
+    #     Aggregate cost computation according to the Quasi-Monte Carlo simulation.
+    #
+    #     :return: aggregate cost empirical pdf, aggregate cost empirical cdf,aggregate cost simulated values.
+    #     :rtype: ``dict``
+    #     """
+    #     if self.d > 1e-05:
+    #         p0 = self.sv.cdf(self.d)
+    #     else:
+    #         p0 = 0
+    #
+    #     logger.info('..QMC Simulating the aggregate distribution..')
+    #
+    #     fqsample = self.fq.rvs(self.nsim, random_state=self.setseed)
+    #     # fss = hfns.sobol_generator(n=self.nsim, dim=1,skip=int(0+self.skip)).flatten()
+    #     # fqsample = self.fq.ppf(fss)
+    #     # sss = hfns.sobol_generator(n=int(np.sum(fqsample) + 1.1*np.ceil(p0 * self.nsim)), dim=1,skip=int(self.nsim+self.skip)).flatten()
+    #     sssgen = qmc.Sobol(d=1, scramble=True,seed=self.setseed)
+    #     sss=sssgen.random(n=int(np.sum(fqsample) + 1.1*np.ceil(p0 * self.nsim))).flatten()
+    #     svsample = self.sv.ppf(sss)
+    #
+    #     if self.d > 1e-05:
+    #         np.random.seed(self.setseed)
+    #         svsample = svsample[svsample > self.d]
+    #         while svsample.shape[0] < self.nsim:
+    #             n0 = np.ceil(p0 * (self.nsim - svsample.shape[0]))
+    #             sssgen = qmc.Sobol(d=1, scramble=True)
+    #             sss = sssgen.random(n=int(1.1*n0)).flatten()
+    #             # sss = hfns.sobol_generator(n=int(1.1*n0), dim=1,skip=self.skip).flatten()
+    #             svsample = np.concatenate((svsample, self.sv.ppf(sss)))
+    #             svsample = svsample[svsample > self.d]
+    #             # timeel_=round((datetime.now()-now).seconds/60)
+    #
+    #             # if timeel_%3 == 0 and timeel_!=0 and count_ ==0:
+    #             #     logger.warning('Pay attention: when a deductible is present, QMC simulation can be very slow.\n %s minutes elapsed, already'%str((datetime.now()-now).seconds/60))
+    #             #     count_=count_
+    #
+    #         svsample = svsample - self.d
+    #
+    #     if self.u < float('inf'):
+    #         svsample = np.minimum(svsample, self.u)
+    #
+    #     # random.seed(setseed) elimino: trovo seed nelle funzioni dopo
+    #     # fss = hfns.sobol_generator(n=self.nsim, dim=1).flatten() #sobol_seq.i4_sobol_generate(1, self.nsim)
+    #     # update frequency parameters
+    #     # sample frequency
+    #     # fqsample = self.fq.ppf(fss)
+    #     # sss = hfns.sobol_generator(n=int(np.sum(fqsample)), dim=1).flatten()
+    #     # sss = sobol_seq.i4_sobol_generate(1, int(np.sum(fqsample))).flatten()
+    #
+    #     # svsample = self.sv.ppf(sss)
+    #     cs = np.cumsum(fqsample).astype(int)
+    #
+    #     if fqsample[0:1] == 0:
+    #         xsim = np.array([0])
+    #     else:
+    #         xsim = np.array([np.sum(svsample[0:cs[0]])])
+    #
+    #     for i in range(0, self.nsim - 1):
+    #         if cs[i] == cs[i + 1]:
+    #             # xsim.append(0)
+    #             xsim=np.concatenate((xsim,[0]))
+    #         else:
+    #             # xsim.append(np.sum(svsample[cs[i]:cs[i + 1]]))
+    #             xsim = np.concatenate((xsim, [np.sum(svsample[cs[i]:cs[i + 1]])]))
+    #
+    #     # cs = np.cumsum(fqsample).astype('int')
+    #     #
+    #     # if fqsample[0:1] == 0:
+    #     #     xsim = [0]
+    #     # else:
+    #     #     xsim = [np.sum(svsample[0:int(cs[0])])]
+    #     #
+    #     # for i in range(0, self.nsim - 1):
+    #     #     if cs[i] == cs[i + 1]:
+    #     #         xsim.append(0)
+    #     #     else:
+    #     #         xsim.append(np.sum(svsample[int(cs[i]):int(cs[i + 1])]))
+    #
+    #     logger.info('..QMC Simulation completed..')
+    #
+    #     x_, ecdf = hfns.ecdf(xsim)
+    #     epdf = np.repeat(1 / self.nsim, self.nsim)
+    #
+    #     return {'epdf': epdf,
+    #             'ecdf':ecdf,
+    #             'sequence':x_}
 
         # return {'epdf': epdf(np.arange(0, np.max(xsim))),'ecdf':ECDF(xsim).y[1:],'sequence':ECDF(xsim).x[1:]}
 
     def empiricalmoments(self,central=False,order=1):
+
         """
         Empirical moments computed on the aggregate cost.
 
@@ -914,3 +1112,4 @@ class LossModel(Frequency,Severity):
 
 
 
+
