@@ -8,10 +8,11 @@ from twiggy import quick_setup,log
 quick_setup()
 logger = log.name('lossreserve')
 
-
 class LossReserve:
     """
-    Class to compute the loss reserve. The available models are the Fisher-Lange model and the collective risk model.
+    Class to compute the loss reserve. The available models are the deterministic Fisher-Lange and the collective risk model.
+    Input company data must be ``numpy.ndarray`` data on numbers and payments must be in triangular form:
+    two dimensional ``numpy.ndarray`` with shape (I,J) with I=J.
 
     :param tail: set it to True when the tail estimate is required. Default False.
     :type tail: ``bool``
@@ -45,6 +46,8 @@ class LossReserve:
     :Keyword Arguments:
         * *ip_tr* = (``numpy.ndarray``) --
             Incremental payments triangle
+        * *cumulatives_tr* = (``numpy.ndarray``) --
+            Cumulative payments triangle
         * *cp_tr* = (``numpy.ndarray``) --
             Cased payments triangle
         * *in_tr* = (``numpy.ndarray``) --
@@ -62,6 +65,7 @@ class LossReserve:
                  mixingfpar=None,
                  czj=None,
                  setseed=42,
+                 cumulatives_tr=None,
                  mixingspar=None,**kwargs):
 
         self.tail=tail
@@ -80,22 +84,23 @@ class LossReserve:
         self.czj_t = np.tile(self.czj, self.j).reshape(self.j, -1)
 
         #triangles
-        self.ip_tr= kwargs['incremental_payments']
+        self.ip_tr = kwargs['incremental_payments']
         self.cp_tr = kwargs['cased_payments']
-        self.in_tr= kwargs['incurred_number']
+        self.in_tr = kwargs['incurred_number']
         self.cn_tr = kwargs['cased_number']
-        self.ap_tr= None #Il triangolo dei pagamenti medi è costruito nelle proprietà senza effetivamente definire un metodo
+        self.ap_tr = None
+        self.cumulatives_tr = cumulatives_tr
 
         #variables and other objects
         if self.reserving_method == 'crm':
-            self.ntr_sim=ntr_sim
-            self.setseed=setseed
-            self.mixingfpar=mixingfpar
-            self.mixingspar=mixingspar
+            self.ntr_sim = ntr_sim
+            self.setseed = setseed
+            self.mixingfpar = mixingfpar
+            self.mixingspar = mixingspar
             self.gamma1 = stats.gamma(**self.mixingfpar)
             self.gamma2 = stats.gamma(**self.mixingspar)
-            self.gamma3=stats.gamma
-            self.pois=stats.poisson
+            self.gamma3 = stats.gamma
+            self.pois = stats.poisson
 
         #attributes with opportunities of not standard customization
         if not custom_alphas == None:
@@ -109,13 +114,13 @@ class LossReserve:
             self.ssFl_= self.__SsComputer()
 
         # results
-        self.ss_tr=self.__SsTriangle()
-        self.predicted_Inumbers = self.__fillNumbers()
+        self.ss_tr=self.__SsTriangle() #triangle of settlement speed
+        self.predicted_Inumbers = self.__fillNumbers() #individual payments triangle
 
-        self.flreserve=self.__Flreserve()
+        self.flreserve=self.__Flreserve() #fisher-lange reserve
 
         if self.reserving_method=='crm':
-            self.crmreserve,self.msep,self.skewness=self.__StochasticCRM()
+            self.crmreserve,self.msep,self.skewness=self.__StochasticCRM() #crm reserving: value of the reserve, mean-squared error of prediction and skewness.
 
     @property
     def tail(self):
@@ -132,7 +137,7 @@ class LossReserve:
 
     @reserving_method.setter
     def reserving_method(self, var):
-        assert var.lower() in ['fisherlange','crm'], logger.error("%r is not supported from our package"%var)
+        assert var.lower() in ['chl','fisherlange','crm'], logger.error("%r is not supported from our package"%var)
         assert isinstance(var,str), logger.error("%r must be a string"%var)
         if not var.lower() == var:
             logger.warning("What did you mean with %r? 'reservingmethod' is set to %r." % (var, var.lower()))
@@ -177,7 +182,7 @@ class LossReserve:
 
     @czj.setter
     def czj(self, var):
-        if self.reserving_method == 'fisherlange':
+        if self.reserving_method in ['chl','fisherlange']:
             self.__czj = None
         else:
             if var is None:
@@ -204,30 +209,83 @@ class LossReserve:
 
     @ip_tr.setter
     def ip_tr(self, var):
-        var = np.array(var).astype(float)
-        assert type(var) == np.ndarray, logger.error('The Incremental Payments triangle must be a two dimensional numpy.ndarray')
-        assert var.shape[0] == var.shape[1], logger.error('The Incremental Payments triangle must be a two dimensional with same shape on both dimensions. \n The triangle shape is: %s'%str(var.shape))
+        if self.reserving_method in ['chl']:
+            if var is None:
+                self.__ip_tr = var
+            else:
+                var = np.array(var).astype(float)
+                assert type(var) == np.ndarray, logger.error(
+                    'The Incremental Payments triangle must be a two dimensional numpy.ndarray')
+                assert var.shape[0] == var.shape[1], logger.error(
+                    'The Incremental Payments triangle must be a two dimensional with same shape on both dimensions. \n The triangle shape is: %s' % str(
+                        var.shape))
 
-        nans=np.isnan(var) #sia con np.nan di numpy che con NaN di python te li trova. Ricordiamoci che è un punto delicato
+                nans = np.isnan(
+                    var)  # sia con np.nan di numpy che con NaN di python te li trova. Ricordiamoci che è un punto delicato
+
+                if np.sum(nans) > 0:
+                    assert np.min(self.ix[nans]) > self.j, logger.error(
+                        'Check your Incremental Payments input.\n Not valid values in the upper triangle.')
+                self.__ip_tr = var
+        else:
+            var = np.array(var).astype(float)
+            assert type(var) == np.ndarray, logger.error('The Incremental Payments triangle must be a two dimensional numpy.ndarray')
+            assert var.shape[0] == var.shape[1], logger.error('The Incremental Payments triangle must be a two dimensional with same shape on both dimensions. \n The triangle shape is: %s'%str(var.shape))
+
+            nans=np.isnan(var) #sia con np.nan di numpy che con NaN di python te li trova. Ricordiamoci che è un punto delicato
+
+            if np.sum(nans) > 0:
+                assert np.min(self.ix[nans]) > self.j, logger.error('Check your Incremental Payments input.\n Not valid values in the upper triangle.')
+            self.__ip_tr = var
+
+    @property
+    def cumulatives_tr(self):
+        return self.__cumulatives_tr
+
+    @cumulatives_tr.setter
+    def cumulatives_tr(self, var):
+        if self.reserving_method in ['chl']:
+            if self.ip_tr is None:
+                var = np.array(var).astype(float)
+                assert type(var) == np.ndarray, logger.error(
+                    'The Incremental Payments triangle must be a two dimensional numpy.ndarray')
+                assert var.shape[0] == var.shape[1], logger.error(
+                    'The Incremental Payments triangle must be a two dimensional with same shape on both dimensions. \n The triangle shape is: %s' % str(
+                        var.shape))
+
+                nans = np.isnan(
+                    var)  # sia con np.nan di numpy che con NaN di python te li trova. Ricordiamoci che è un punto delicato
+
+                if np.sum(nans) > 0:
+                    assert np.min(self.ix[nans]) > self.j, logger.error(
+                        'Check your Incremental Payments input.\n Not valid values in the upper triangle.')
+                self.__cumulatives_tr = var
+            else:
+                cumuls=self.ip_tr.copy()
+                for i in range(1, cumuls.shape[1]):
+                    cumuls[:, i] = cumuls[:, i] + cumuls[:, i - 1]
+                self.__cumulatives_tr = cumuls
+        else:
+            self.__cumulatives_tr = None
 
-        if np.sum(nans) > 0:
-            assert np.min(self.ix[nans]) > self.j, logger.error('Check your Incremental Payments input.\n Not valid values in the upper triangle.')
-        self.__ip_tr = var
     @property
     def cp_tr(self):
         return self.__cp_tr
 
     @cp_tr.setter
     def cp_tr(self, var):
-        var = np.array(var).astype(float)
-        assert type(var) == np.ndarray, logger.error('The Cased Payments triangle must be a two dimensional numpy.ndarray')
-        assert var.shape[0] == var.shape[1], logger.error('The Cased Payments triangle must be a two dimensional with same shape on both dimensions. \n The triangle shape is: %s'%str(var.shape))
+        if self.reserving_method in ['fisherlange', 'crm']:
+            var = np.array(var).astype(float)
+            assert type(var) == np.ndarray, logger.error('The Cased Payments triangle must be a two dimensional numpy.ndarray')
+            assert var.shape[0] == var.shape[1], logger.error('The Cased Payments triangle must be a two dimensional with same shape on both dimensions. \n The triangle shape is: %s'%str(var.shape))
 
-        nans=np.isnan(var) #sia con np.nan di numpy che con NaN di python te li trova. Ricordiamoci che è un punto delicato
+            nans=np.isnan(var) #sia con np.nan di numpy che con NaN di python te li trova. Ricordiamoci che è un punto delicato
 
-        if np.sum(nans) > 0:
-            assert np.min(self.ix[nans]) > self.j, logger.error('Check your Cased Payments input.\n Not valid values in the upper triangle.')
-        self.__cp_tr = var
+            if np.sum(nans) > 0:
+                assert np.min(self.ix[nans]) > self.j, logger.error('Check your Cased Payments input.\n Not valid values in the upper triangle.')
+            self.__cp_tr = var
+        else:
+            self.__cp_tr = var
 
     @property
     def in_tr(self):
@@ -235,15 +293,18 @@ class LossReserve:
 
     @in_tr.setter
     def in_tr(self, var):
-        var=np.array(var).astype(float)
-        assert type(var) == np.ndarray, logger.error('The Incurred Number triangle must be a two dimensional numpy.ndarray')
-        assert var.shape[0] == var.shape[1], logger.error('The Incurred Number triangle must be a two dimensional with same shape on both dimensions. \n The triangle shape is: %s'%str(var.shape))
+        if self.reserving_method in ['fisherlange', 'crm']:
+            var=np.array(var).astype(float)
+            assert type(var) == np.ndarray, logger.error('The Incurred Number triangle must be a two dimensional numpy.ndarray')
+            assert var.shape[0] == var.shape[1], logger.error('The Incurred Number triangle must be a two dimensional with same shape on both dimensions. \n The triangle shape is: %s'%str(var.shape))
 
-        nans=np.isnan(var) #sia con np.nan di numpy che con NaN di python te li trova. Ricordiamoci che è un punto delicato
+            nans=np.isnan(var) #sia con np.nan di numpy che con NaN di python te li trova. Ricordiamoci che è un punto delicato
 
-        if np.sum(nans) > 0:
-            assert np.min(self.ix[nans]) > self.j, logger.error('Check your Incurred Number input. \n Not valid values in the upper triangle.')
-        self.__in_tr = var
+            if np.sum(nans) > 0:
+                assert np.min(self.ix[nans]) > self.j, logger.error('Check your Incurred Number input. \n Not valid values in the upper triangle.')
+            self.__in_tr = var
+        else:
+            self.__in_tr = None
 
     @property
     def cn_tr(self):
@@ -251,15 +312,18 @@ class LossReserve:
 
     @cn_tr.setter
     def cn_tr(self, var):
-        var = np.array(var).astype(float)
-        assert type(var) == np.ndarray, logger.error('The Cased Number triangle must be a two dimensional numpy.ndarray')
-        assert var.shape[0] == var.shape[1], logger.error('The Cased Number triangle must be a two dimensional with same shape on both dimensions. \n The triangle shape is: %s'%str(var.shape))
+        if self.reserving_method in ['fisherlange', 'crm']:
+            var = np.array(var).astype(float)
+            assert type(var) == np.ndarray, logger.error('The Cased Number triangle must be a two dimensional numpy.ndarray')
+            assert var.shape[0] == var.shape[1], logger.error('The Cased Number triangle must be a two dimensional with same shape on both dimensions. \n The triangle shape is: %s'%str(var.shape))
 
-        nans=np.isnan(var) #sia con np.nan di numpy che con NaN di python te li trova. Ricordiamoci che è un punto delicato
+            nans=np.isnan(var) #sia con np.nan di numpy che con NaN di python te li trova. Ricordiamoci che è un punto delicato
 
-        if np.sum(nans) > 0:
-            assert np.min(self.ix[nans]) > self.j, logger.error('Check your Cased Number input.\n Not valid values in the upper triangle.')
-        self.__cn_tr = var
+            if np.sum(nans) > 0:
+                assert np.min(self.ix[nans]) > self.j, logger.error('Check your Cased Number input.\n Not valid values in the upper triangle.')
+            self.__cn_tr = var
+        else:
+            self.__cn_tr = None
 
     @property
     def ntr_sim(self):
@@ -293,7 +357,7 @@ class LossReserve:
 
     @mixingfpar.setter
     def mixingfpar(self, var):
-        if not self.reserving_method == "fisherlange":
+        if self.reserving_method == "crm":
             try:
                 assert isinstance(var,dict), logger.error("The frequency mixing parameters for the crm reserving model must be provided as a dictionary")
                 assert all(item in list(var.keys()) for item in ["scale","a"]), logger.error("The mixing frequency parameters for the crm reserving model must be provided as 'a' and 'scale'. See: https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.gamma.html")
@@ -309,7 +373,7 @@ class LossReserve:
 
     @mixingspar.setter
     def mixingspar(self, var):
-        if not self.reserving_method == "fisherlange":
+        if self.reserving_method == "crm":
             try:
                 assert isinstance(var, dict), logger.error(
                     "The severity mixing parameters for the crm reserving model must be provided as a dictionary")
@@ -572,7 +636,6 @@ class LossReserve:
     def SSPlot(self,start_=0):
         """
         It plots the settlement speed vector for each accident year.
-
         :param start_: starting accident year from which to plot.
         :type start_: ``int``
         """
@@ -588,7 +651,6 @@ class LossReserve:
     def AverageCostPlot(self):
         """
         It plots the mean average cost for each development year.
-
         """
         x_ = np.arange(0, self.j+self.tail)
         plt.title('Plot of the Average Cost (mean of each DY, data and predictions)')
@@ -601,7 +663,6 @@ class LossReserve:
     def AlphaPlot(self):
         """
         It plots the Fisher-Lange alpha.
-
         """
         x_ = np.arange(0, self.j+self.tail-1)
         plt.title('Plot of Alpha')
